import requests
import logging
from PIL import Image
import io
import os
from ic_slide.time_utils import timeit
from .auth import get_access_token
import numpy as np
from .api_urls import get_metadata_url,\
    get_tile_url
from .config import get_config
import functools
import multiprocessing as mp

logger = logging.getLogger(__name__)


class SlideCloudStorage:

    def __init__(self) -> None:
        self.metadata_url = get_metadata_url()
        self.tile_url = get_tile_url()

    def get_metadata(self, slide_id):
        if not slide_id:
            raise ValueError('Invalid empty slide id')
        url = f'{self.metadata_url}/{slide_id}'
        access_token = get_access_token()
        headers = {"Authorization": f"Bearer {access_token}"}
        metadata = requests.get(url,
                                headers=headers).json()
        if 'error' in metadata:
            raise ValueError(metadata['error']['message'])

        result = Metadata(metadata)
        result.SlideId = slide_id
        return result

    @timeit
    @functools.lru_cache(maxsize=get_config('TILE_CACHE_MAX_SIZE'))
    def get_tile(self, slide_id, row, column, layer, lod):
        url = f'{self.tile_url}?id={slide_id}&Row={row}&Column={column}&Layer={layer}&LODLevel={lod}'
        access_token = get_access_token()
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(url, headers=headers)

        if response.status_code >= 200 and response.status_code <= 299:
            tile_url = response.content
        else:
            raise Exception(
                f"Can not get tile, {response.reason} {response.content}")
        response = requests.get(tile_url)
        if response.status_code >= 200 and response.status_code <= 299:
            return response.content
        return None


class Metadata(object):
    SlideId = ''
    Name = ''
    LayerCount = 0
    MinimumLODLevel = 0
    MaximumLODLevel = 0
    LODGaps = []
    QuickHash = ''
    Vendor = ''
    Version = None
    Comments = ''
    BackgroundColor = None
    HorizontalTileCount = 0
    VertialTileCount = 0
    LayerCount = 0
    TileSize = {}
    ContentRegion = {}
    HorizontalResolution = 0
    VeriticalResolution = 0
    AdditionalData = {}

    def __init__(self, metadata_dict):
        self.__dict__ = metadata_dict

    def get_default_layer(self):
        return 1 if self.LayerCount == 1 else 0

    def get_lod_to_world_scale(self, lod):
        if(lod < 0 or lod > self.MaximumLODLevel):
            raise ValueError(
                f'load {lod} out of range [0-{self.MaximumLODLevel}]')

        scale = 1.0

        if lod != 0:
            for gap in self.LODGaps:
                scale /= gap

        return scale


class Slide:
    def __init__(self, metadata, slide_storage):
        self.metadata = metadata
        self._slide_storage = slide_storage

    def read_region(self, x, y, width, height, layer=None, lod=None):
        final_layer = layer if layer else self.metadata.get_default_layer()
        final_lod = lod if lod else self.metadata.MinimumLODLevel
        tile_width = self.metadata.TileSize['Width']
        tile_height = self.metadata.TileSize['Height']
        tile_index_left = int(x / tile_width)
        tile_index_top = int(y / tile_height)
        tile_index_right = int((x + width - 1) / tile_width)
        tile_index_bottom = int((y + height - 1)/tile_height)

        new_image = Image.new('RGB', (width, height), (255, 255, 255))

        offset_y = y
        # must +1 to include stop
        for row in range(tile_index_top, tile_index_bottom+1):
            tile_top = row * tile_height
            tile_bottom = (row+1) * tile_height
            offset_x = x
            area_y = offset_y - tile_top
            area_height = min(tile_bottom - offset_y, height - offset_y + y)
            # must +1 to include stop
            for column in range(tile_index_left, tile_index_right+1):
                tile_left = column * tile_width
                tile_right = (column+1) * tile_width
                area_x = offset_x - tile_left
                tile_data = self._slide_storage.get_tile(
                    self.metadata.SlideId, int(row), int(column), final_layer, final_lod)
                logger.debug(
                    f'get tile {row}-{column}-{final_layer}-{final_lod} data')
                area_width = min(tile_right - offset_x, width - offset_x + x)
                if tile_data:
                    image = Image.open(io.BytesIO(tile_data))
                    image_valid_area = (area_x,
                                        area_y,
                                        area_width + area_x,
                                        area_height + area_y)
                    cropped_image = image.crop(image_valid_area)
                    # cropped_image.save(f"{image_valid_area}.jpg")
                    new_image.paste(cropped_image, (offset_x - x, offset_y-y))
                offset_x += area_width
            offset_y += area_height
        return new_image


def open_slide(slide_id):
    '''
    Open a cloud slide of coriander.

    Args:
        slide_id; the id of slide.

    Returns:
        slide (Slide): the slide.
    '''
    logger.info(f"open slide with slide id {slide_id}")
    slide_storage = SlideCloudStorage()
    metadata = slide_storage.get_metadata(slide_id)
    return Slide(metadata, slide_storage)


def enumerate_tiles(slide_id,
                    begin,
                    stop,
                    stride=(400, 400),
                    size=(512, 512),
                    auto_scaling=True):
    '''
    Get tiles of slide id with begin and end.

    Args:
        begin (list[2]): the begin location to the slide.

        end (list[2]): the end location to the slide.

        stride (tuple(int, int)): the stride to get tiles in Vertical/ Horizontal.

        size (tuple(int,int)): the size of tile image.

        auto-scaling (boolean): should unify slides in same standard by mpp.

    Returns:
        tiles (Generator[Image]): the tile images
    '''
    callbacks = enumerate_tile_funcs(
        slide_id, begin, stop, stride, size, auto_scaling)
    for callback in callbacks:
        yield callback()


def enumerate_tile_funcs(slide_id,
                         begin,
                         stop,
                         stride=(400, 400),
                         size=(512, 512),
                         auto_scaling=True):
    slide = open_slide(slide_id)
    logger.info(f"Enumerate slide {slide_id}  tiles.")
    return __enumerate_slide_tile_funcs(slide, begin, stop, stride, size, auto_scaling)


def enumerate_tiles_count(slide_id,
                          begin,
                          stop,
                          stride=(400, 400),
                          size=(512, 512),
                          auto_scaling=True):
    slide = open_slide(slide_id)
    logger.info(f"Enumerate slide {slide_id}  tiles.")
    begin, stop, stride, size, step, _ = __get_enumrate_parameters(
        slide, begin, stop, stride, size, auto_scaling)

    return step[0] * step[1]


class _StopToken:
    pass


def produce_regions(index, tile_queue, tile_callbacks):
    try:
        for tile_callback in tile_callbacks:
            tile = tile_callback()
            tile_queue.put(tile)
    except Exception as e:
        logger.warn(f"tile producer {index} error {e}")
    finally:
        tile_queue.put(_StopToken())


def enumerate_content_tiles_async(slide_id, stride=(400, 400), size=(512, 512), auto_scaling=True, number_of_producer=16):
    tile_callbacks = list(enumerate_content_tile_funcs(slide_id,
                                                       stride,
                                                       size,
                                                       auto_scaling))
    if number_of_producer <= 0:
        raise ValueError(number_of_producer)

    tiles_count = len(tile_callbacks)
    split_sections = [int(tiles_count /
                          number_of_producer) * i for i in range(number_of_producer)]
    workers_tile_callbacks = np.split(
        tile_callbacks, split_sections[1:])

    ctx = mp.get_context('fork')
    workers = []
    tile_queue = mp.Queue(number_of_producer*8)
    for index, worker_tile_callbacks in enumerate(workers_tile_callbacks):
        process = ctx.Process(target=produce_regions, args=(
            index,
            tile_queue,
            worker_tile_callbacks))
        workers.append(process)

    for worker in workers:
        worker.start()

    done_worker_count = 0
    while True:
        tile = tile_queue.get()
        if isinstance(tile, _StopToken):
            done_worker_count += 1
        if done_worker_count == number_of_producer:
            break
        yield tile
    tile_queue.close()
    for worker in workers:
        worker.join()


def enumerate_content_tiles(slide_id,
                            stride=(400, 400),
                            size=(512, 512),
                            auto_scaling=True,
                            is_async=False):
    '''
    Get tiles in the content area of slide

    Args:
        stride (tuple(int, int)): the stride to get tiles in Vertical/ Horizontal.

        size (tuple(int,int)): the size of tile image.

        auto-scaling (boolean): should unify slides in same standard by mpp.

        is_async(boolean): represent utilize multiple cpus cores to load tiles

    Returns:
        tiles (Generator[Image]): the tile images
    '''

    if is_async:
        cpu_count = os.cpu_count()
        return enumerate_content_tiles_async(slide_id, stride, size, auto_scaling, cpu_count)
    else:
        def enumerate_tile_callbacks():
            callbacks = enumerate_content_tile_funcs(slide_id,
                                                     stride,
                                                     size,
                                                     auto_scaling)
            for callback in callbacks:
                yield callback()
        return enumerate_tile_callbacks()


def enumerate_content_tile_funcs(slide_id, stride=(400, 400), size=(512, 512), auto_scaling=True):
    slide = open_slide(slide_id)
    logger.info(f"Enumerate slide {slide_id} content tiles .")
    begin = int(slide.metadata.ContentRegion['X']), int(
        slide.metadata.ContentRegion['Y'])

    stop = begin[0] + int(slide.metadata.ContentRegion["Width"]
                          ), begin[1]+int(slide.metadata.ContentRegion["Height"])
    return __enumerate_slide_tile_funcs(slide, begin, stop, stride, size, auto_scaling)


def enumerate_content_tiles_count(slide_id,
                                  stride=(400, 400),
                                  size=(512, 512),
                                  auto_scaling=True):
    slide = open_slide(slide_id)
    logger.info(f"Enumerate slide {slide_id} content tiles .")
    begin = int(slide.metadata.ContentRegion['X']), int(
        slide.metadata.ContentRegion['Y'])

    stop = begin[0] + int(slide.metadata.ContentRegion["Width"]
                          ), begin[1]+int(slide.metadata.ContentRegion["Height"])

    begin, stop, stride, size, step, scale = __get_enumrate_parameters(
        slide, begin, stop, stride, size, auto_scaling)

    return step[0] * step[1]


def __get_enumrate_parameters(slide,
                              begin,
                              stop,
                              stride=(400, 400),
                              size=(512, 512),
                              auto_scaling=True):
    if not slide:
        raise ValueError("Can not enumerate tiles with empty slide.")

    if not begin or len(begin) != 2:
        raise ValueError(f"Invalid begin value {begin}")

    if not stop or len(stop) != 2:
        raise ValueError(f"Invalid stop value {stop}.")

    if not stride or len(stride) != 2:
        raise ValueError(f"Invalid stride value {stride}.")

    if not size or len(size) != 2:
        raise ValueError(f"Invalide size value {size}.")
    scale = [1, 1]
    if auto_scaling:
        scale = __get_slide_scale(slide)
    begin = np.array(begin)
    stop = np.array(stop)
    stride = (np.array(stride) * scale).astype(int)
    size = (np.array(size) * scale).astype(int)

    step = ((stop - begin - size)/stride+1).astype(int)
    return (begin, stop, stride, size, step, scale)


def __enumerate_slide_tile_funcs(slide, begin, stop, stride, size, auto_scaling=True):
    begin, stop, stride, scaling_size, step, scale = __get_enumrate_parameters(
        slide, begin, stop, stride, size, auto_scaling)
    for row in range(step[1]):
        for col in range(step[0]):
            yield __tile_func_wrapper(slide, begin, stride, size, scaling_size, col, row)


def __tile_func_wrapper(slide, begin, stride, size, scaling_size, col, row):
    def func():
        x0, y0 = (begin + np.array((col, row))
                  * stride).astype(int)
        image = slide.read_region(
            x0, y0, scaling_size[0], scaling_size[1], None, None)
        if image.width != size[0] or image.height != size[1]:
            image = image.resize(size, Image.ANTIALIAS)
        return {
            'location': [x0, y0],
            'image': image,
            'size': list(size),
            'step': [col, row]
        }
    return func


def __get_slide_scale(slide):
    mpp_x = get_config('MICRON_PER_PIXEL_X')
    mpp_y = get_config('MICRON_PER_PIXEL_Y')
    slide_metadata = slide.metadata
    if slide_metadata.HorizontalResolution and \
            slide_metadata.HorizontalResolution > 0 and \
            slide_metadata.VerticalResolution and \
            slide_metadata.VerticalResolution > 0:
        return mpp_x/slide_metadata.HorizontalResolution, \
            mpp_y/slide_metadata.VerticalResolution
    return [1, 1]



from .config import get_config, is_client_grant


def urljoin(*args):
    """
    Joins given arguments into an url. Trailing but not leading slashes are
    stripped for each argument.
    """
    return "/".join(map(lambda x: str(x).rstrip('/'), args))


def get_storage_index_files_api_url():
    host_url = get_config("STORAGEINDEX_URL")
    return urljoin(
        host_url, "api/file-entry/all-organization-files"
    )


def get_storage_index_request_permission_token_api_url():
    if is_client_grant():
        raise Exception(
            "We can not access storage index with client grant type.")
    host_url = get_config("STORAGEINDEX_URL")
    return urljoin(
        host_url, "api/entry/request-permission-token"
    )


def get_asset_file_entry_api_url():
    if is_client_grant():
        raise Exception(
            "We can not access asset service with client grant type.")
    host_url = get_config("ASSETSERVICE_URL")
    return urljoin(
        host_url, "api/asset-service/file-entry"
    )


def get_metadata_url():
    host_url = get_config("SLIDECLOUD_URL")
    return urljoin(host_url,
                   "api/app/slideClientOnly/slideMetadata")


def get_tile_url():
    host_url = get_config("SLIDECLOUD_URL")
    return urljoin(host_url,
                   "api/app/slideClientOnly/tileUrl")



from .slide_source import open_slide, enumerate_tiles,\
    enumerate_tiles_count, \
    enumerate_tile_funcs, \
    enumerate_content_tiles, \
    enumerate_content_tiles_count,\
    enumerate_content_tile_funcs, \
    enumerate_content_tiles_async

from .slide_index import get_slide_entries,\
    get_distinct_slide_ids,\
    get_slide_entry_annotations

from .config import get_config, \
    update_config, use_client_grant, \
    use_password_grant



import logging
from getpass import getpass

logger = logging.Logger(__name__)

SLIDE_CONFIG = {
    'AUTH_URL': 'http://private.intemedic.com:6001',
    'SLIDECLOUD_URL': 'http://private.intemedic.com:6002',
    'STORAGEINDEX_URL': 'http://private.intemedic.com:6003',
    'ASSETSERVICE_URL': 'http://private.intemedic.com:6004',
    'MICRON_PER_PIXEL_X': 0.35093510,
    'MICRON_PER_PIXEL_Y': 0.35150376,
    'TILE_CACHE_MAX_SIZE': 1024
}


def update_config(**kwargs):
    new_config = SLIDE_CONFIG.update(kwargs)
    logger.info(f"new config is {SLIDE_CONFIG}")
    return new_config


def use_password_grant(username=None, password=None):
    if username:
        USERNAME = username
    else:
        USERNAME = input("Please enter the user name of coriander: ")

    if password:
        PASSWORD = password
    else:
        PASSWORD = getpass(f"Please enter the password of {USERNAME}: ")

    logger.info(f"use password grant type to access private cloud.")
    new_config = SLIDE_CONFIG.update(
        USERNAME=USERNAME,
        PASSWORD=PASSWORD,
        SCOPE='AlgorithmService StorageIndex SlideCloudStorage AuthServer',
        CLIENT_ID='coriander',
        CLIENT_SECRET='secret',
        GRANT_TYPE='password')
    logger.info(f"new config is {SLIDE_CONFIG}")
    return new_config


def use_client_grant():
    new_config = SLIDE_CONFIG.update(
        GRANT_TYPE='client_credentials',
        SCOPE='StorageIndex SlideCloudStorage AuthServer',
        CLIENT_SECRET='secret',
        CLIENT_ID='coriander-service')
    logger.info(f"new config is {SLIDE_CONFIG}")
    return new_config


def is_client_grant():
    return SLIDE_CONFIG['GRANT_TYPE'] == 'client_credentials'


def get_grant_data():
    if is_client_grant():
        return {
            "grant_type": SLIDE_CONFIG['GRANT_TYPE'],
            "scope": SLIDE_CONFIG['SCOPE'],
            "client_id": SLIDE_CONFIG['CLIENT_ID'],
            "client_secret": SLIDE_CONFIG['CLIENT_SECRET']
        }
    return {
        "grant_type": SLIDE_CONFIG['GRANT_TYPE'],
        "scope": SLIDE_CONFIG['SCOPE'],
        "client_id": SLIDE_CONFIG['CLIENT_ID'],
        "client_secret": SLIDE_CONFIG['CLIENT_SECRET'],
        "username": SLIDE_CONFIG['USERNAME'],
        "password": SLIDE_CONFIG['PASSWORD']
    }


def get_config(key: str):
    return SLIDE_CONFIG[key]


use_client_grant()




import time
import logging
logger = logging.getLogger(__file__)


def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            logger.debug(f"{method.__name__}  {(te - ts) * 1000} ms")
        return result
    return timed



import numpy as np
from .auth import get_access_token
import json
import requests
import logging
import base64
import math
from .api_urls import get_storage_index_files_api_url,  \
    get_storage_index_request_permission_token_api_url,\
    get_asset_file_entry_api_url

logger = logging.Logger(__name__)


class AnnotationDto(object):
    '''
    The slide annotation struct.

    Args:
        tags (list(str)): a lot of tag strings.

        comments (str): a comment string to annotation.

        bounds (list[int]): a area with location and size.

        vertices (list[list[int]]): vertices of annotation shape.
    '''

    def __init__(self,
                 tags,
                 comments,
                 bounds,
                 vertices) -> None:
        self.Tags = tags
        self.Comments = comments
        self.Bounds = bounds
        self.Vertices = vertices

    def __repr__(self) -> str:
        return f"AnnotationDto(Tags={self.Tags},"\
            f"Comments={self.Comments},"\
            f"Bounds = {self.Bounds}, "\
            f"Vertices = {self.Vertices})"


class SlideEntryDto:
    '''
    The slide entry of slide in the storage. it contains slide entry informations.

    Args:
        name (str): the name of slide entry.

        pathName (str): the path of slide in the Coriander explorer.

        scheme (str): fixed with 'slide'

        storageId (str): Important it represent identity of slide. We can open slide with it.

        creatorId (str): creator of slide entry.

        creationTime (date): creation time of slide entry.

        lastModifierId (str): last modifier of slide entry.

        lastModificationTime (date): last modification time of slide entry.
    '''

    def __init__(self, id,
                 name,
                 pathName,
                 scheme,
                 storageId,
                 creatorId,
                 creationTime,
                 lastModifierId,
                 lastModificationTime):
        self.Id = id
        self.Name = name
        self.PathName = pathName
        self.Scheme = scheme
        self.StorageId = storageId
        self.CreatorId = creatorId
        self.CreationTime = creationTime
        self.LastModifierId = lastModifierId
        self.LastModificationTime = lastModificationTime

    def __repr__(self) -> str:
        return f"FileDto(Id={self.Id}, \
            Name={self.Name}, \
            PathName={self.PathName}, \
            Scheme={self.Scheme}, \
            StorageId={self.StorageId}, \
            CreatorId={self.CreatorId}, \
            CreationTime={self.CreationTime}, \
            LastModifierId={self.LastModifierId}, \
            LastModificationTime={self.LastModificationTime}, \
            )"


def _api_organization_files(filter_by_path=None,
                            filter_by_name=None,
                            filter_by_begin_time=None,
                            filter_by_end_time=None):
    access_token = get_access_token()
    headers = {"Authorization": f"Bearer {access_token}"}
    params = {
        "PathName": filter_by_path,
        "Name": filter_by_name,
        "CreationTimeRegion": filter_by_begin_time,
        "CreationTimeEnd": filter_by_end_time
    }
    logger.debug(f"enumerate slide entries with conditions: {params}.")
    response = requests.get(get_storage_index_files_api_url(),
                            params=params,
                            headers=headers)
    if response.status_code < 200 or response.status_code >= 300:
        raise Exception(
            f"Can not get slide entries, {response.reason} {response.content}")
    content = json.loads(response.content)
    return [SlideEntryDto(**item) for item in content]


def _api_request_permission_token(entry_id) -> str:
    if not entry_id:
        raise ValueError(f"Invalid empty entry id.")
    access_token = get_access_token()
    params = {"EntryId": entry_id}
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.post(get_storage_index_request_permission_token_api_url(),
                             params=params,
                             headers=headers)
    if response.status_code < 200 or response.status_code >= 300:
        raise Exception(
            f"Can not get slide indices, {response.reason} {response.content}")
    return response.content


def _api_get_file_entry_asset(entry_id,
                              asset_file_name,
                              permission_token) -> bytes:
    if not entry_id:
        raise ValueError("Invalid empty entry id.")
    if not asset_file_name:
        raise ValueError("Invalid empty asset file name.")
    if not permission_token:
        raise ValueError("Invalid empty permission token.")
    access_token = get_access_token()
    params = {
        "FileEntryId": entry_id,
        "AssetFileName": asset_file_name,
        "PermissionToken": permission_token
    }
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.get(get_asset_file_entry_api_url(),
                            params=params,
                            headers=headers)
    if response.status_code < 200 or response.status_code >= 300:
        raise Exception(
            f"Can not get slide entries, {response.reason} {response.content}")
    if len(response.content) == 0:
        return b''
    content = json.loads(response.content)
    return content['bytes']


def get_slide_entry_annotations(entry_id):
    '''
    Get the annotations of slide entry.

    Args:
        entry_id (str): the id of slide entry.

    Returns:
        annotations (list(AnnotationDto)): annotations.
    '''
    asset_annotations_name = "annotations"
    permission_token = _api_request_permission_token(entry_id)
    annotations_bytes = _api_get_file_entry_asset(entry_id,
                                                  asset_annotations_name,
                                                  permission_token)
    if len(annotations_bytes) == 0:
        return []
    annotation_json = json.loads(base64.b64decode(annotations_bytes))
    annotations = [_convert_annotation(item) for item in annotation_json]
    return annotations


def _convert_annotation(csharp_annotation: dict):
    tags = csharp_annotation['Tags']
    comments = csharp_annotation['Comments']
    shape_type = csharp_annotation['Shape']['$type']
    if 'PolygonShape' in shape_type:
        vertices = csharp_annotation['Shape']['Vertices']

        vertices = [[float(i) for i in v.split(',')] for v in vertices]

        return _convert_polygon_annotation(tags, comments, vertices)
    if 'RectangleShape' in shape_type \
            or 'ArrowShape' in shape_type:
        bounds = csharp_annotation['Shape']['Bounds']
        bounds = [int(item) for item in bounds.split(',')]
        return _convert_rectangle_annotation(tags, comments, bounds)
    if 'EllipseShape' in shape_type:
        bounds = csharp_annotation['Shape']['Bounds']
        bounds = [int(item) for item in bounds.split(',')]
        return _convert_ellipse_annotation(tags, comments, bounds)

    raise Exception(f"Unsupported annotation shape {shape_type}.")


def _convert_polygon_annotation(tags, comments, vertices):
    bounds = []
    min = np.amin(vertices, axis=0)
    max = np.amax(vertices, axis=0)
    location = min
    size = max - min
    bounds.extend(location)
    bounds.extend(size)
    return AnnotationDto(tags, comments, bounds, vertices)


def _convert_rectangle_annotation(tags, comments, bounds):
    vertices = []
    x = bounds[0]
    y = bounds[1]
    width = bounds[2]
    height = bounds[3]
    vertices.append((x, y))
    vertices.append((x+width, y))
    vertices.append((x+width, y+height))
    vertices.append((x, y+height))
    return AnnotationDto(tags, comments, bounds, vertices)


def _convert_ellipse_annotation(tags, comments, bounds):
    point_count = 16
    vertices = []
    location = np.array(bounds[:2])
    size = np.array(bounds[2:])
    center = size/2 + location

    for i in range(point_count):
        angle = math.pi * 2 * i/point_count
        vertice = center + \
            ((math.cos(angle), math.sin(angle)) * size/2).astype(int)
        vertices.append(list(vertice))

    return AnnotationDto(tags, comments, bounds, vertices)


def get_slide_entries(filter_by_path=None,
                      filter_by_name=None):
    '''
    Get the entries of slide.

    Args:
        filter_by_path (str): filter slide entries by path name.

        filter_by_name (str): filter slide entries by name.

    Rerturns:
        slide_entries (list(SlideEntryDto)): the slide entries.
    '''

    filter_by_begin_time = None
    filter_by_end_time = None
    return _api_organization_files(filter_by_path,
                                   filter_by_name,
                                   filter_by_begin_time,
                                   filter_by_end_time)


def get_distinct_slide_ids(**kwargs):
    '''
    Get the distinct slide ids.
    '''
    logger.info("enumerate distinct slide ids.")
    entries = get_slide_entries(**kwargs)
    storage_ids = [entry.StorageId for entry in entries if entry]
    return list(set(storage_ids))
